<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daschmi@daschmi.de)
	 * @date: 20.04.23
	 * @time: 13:27
	 */
	
	namespace wpsg;
	
	class wpsg_combination extends \wpsg_model {
		
		static $_tableName = 'WPSG_TBL_PRODUCTS_COMBINATION';
		
		/**
		 * @throws Exception
		 */
		public function load($variant_id) {

			parent::load($variant_id);

			$this->data = $this->db->fetchRow("SELECT O.* FROM `".WPSG_TBL_PRODUCTS_COMBINATION."` AS O WHERE O.`id` = '".wpsg_q($variant_id)."' ");
			$this->data['var_key'] = self::normalizeKey($this->data['var_key']);

			if (($this->data['id']??0) != $variant_id) throw new \wpsg\Exception(wpsg_translate(__('Gesuchte ID:#1# Gefundene ID:#2#', 'wpsg'), $variant_id, $this->data['id']??0));

			return true;

		}
		
		public function toJSON(): array {
			
			$json = $this->data;
			
			$json['id'] = intval($json['id']);
			$json['product_id'] = intval($json['product_id']);
			$json['label'] = $this->getLabel();
			$json['loaded'] = true;
			
			$json['anr'] = $this->getMeta('anr', false, '');
			$json['ean'] = $this->getMeta('ean', false, '');
			$json['mpn'] = $this->getMeta('mpn', false, '');
			$json['pzn'] = $this->getMeta('pzn', false, '');
			$json['sku'] = $this->getMeta('sku', false, '');
			$json['gtin'] = $this->getMeta('gtin', false, '');
			$json['upc'] = $this->getMeta('upc', false, '');
			$json['isbn'] = $this->getMeta('isbn', false, '');
			$json['issn'] = $this->getMeta('issn', false, '');
			$json['isin'] = $this->getMeta('isin', false, '');
			
			return $json;
			
		}
		
		public function getLabel(): string {
			
			return self::getLabelFromKey($this->__get('var_key'));
			
		}
		
		/* static */
		
		public static function getLabelFromKey(string $var_key): string {
			
			$arKey = explode('|', self::normalizeKey($var_key));
			$arLabel = [];
			
			foreach ($arKey as $var_vari) {
				
				list($var_id, $vari_id) = array_map('intval', explode(':', $var_vari));
				
				try { $var_label = \wpsg\wpsg_variant::getInstance($var_id)->getLabel(); } catch (\Exception $e) { }
				try { $vari_label = \wpsg\wpsg_variation::getInstance($vari_id)->getLabel(); } catch (\Exception $e) { }
				
				$arLabel[] = $var_label.':'.$vari_label;
				
			}
			
			return implode(' / ', $arLabel);
			
		}
		
		public static function getByProductAndKey(int $product_id, string $var_key): wpsg_combination {
			
			$exist = self::findByProductAndKey($product_id, $var_key);
			
			if ($exist !== null) return $exist;
			
			$combination_id = $GLOBALS['wpsg_db']->ImportQuery(WPSG_TBL_PRODUCTS_COMBINATION, [
				'product_id' => intval($product_id),
				'var_key' => wpsg_q(self::normalizeKey($var_key))
			]);
			
			return wpsg_combination::getInstance($combination_id);
			
		}
		
		public static function findByProductAndKey(int $product_id, string $var_key): ?wpsg_combination {
			
			$var_key = self::normalizeKey($var_key);
			
			$component_id = intval($GLOBALS['wpsg_db']->fetchOnePrepared("SELECT `id` FROM `".WPSG_TBL_PRODUCTS_COMBINATION."` WHERE `product_id` = %d AND `var_key` = %s ", $product_id, self::normalizeKey($var_key)));
			
			if ($component_id > 0) return wpsg_combination::getInstance($component_id);
			else return null;
			
		}
		
		/**
		 * Sortieren, damit die Reihenfolge die eventuell anders sein kann angeglichen wird
		 *
		 * @param string $var_key
		 * @return string
		 */
		public static function normalizeKey(string $var_key): string {
			
			if (substr($var_key, 0, 3) === 'pv_') $var_key = preg_replace('/^pv_\d+\|/', '', $var_key);
		 
			$arKeys = explode('|', $var_key);
			
			sort($arKeys);
		
			return implode('|', $arKeys);
			
		}
		
		
	}